@@ -95,7 +95,10 @@ sub inject_real_metaclass_for {
   }
 
   # needed to ensure the method body is stable and get things named
-  Sub::Defer::undefer_sub($_) for grep defined, values %methods;
+  $methods{$_} = Sub::Defer::undefer_sub($methods{$_})
+    for
+      grep $_ ne 'new' && defined $methods{$_},
+      keys %methods;
   my @attrs;
   {
     # This local is completely not required for roles but harmless
