@@ -7,21 +7,23 @@ sub AUTOLOAD {
   my ($meth) = (our $AUTOLOAD =~ /([^:]+)$/);
   my $self = shift;
   die "Can't call $meth without object instance"
-    unless ref $self;
+    if !ref $self;
+  die "Can't inflate Moose metaclass with Moo::sification disabled"
+    if $Moo::sification::disabled;
   require Moo::HandleMoose;
   Moo::HandleMoose::inject_real_metaclass_for($self->{name})->$meth(@_)
 }
 sub can {
   my $self = shift;
   return $self->SUPER::can(@_)
-    unless ref $self;
+    if !ref $self or $Moo::sification::disabled;
   require Moo::HandleMoose;
   Moo::HandleMoose::inject_real_metaclass_for($self->{name})->can(@_)
 }
 sub isa {
   my $self = shift;
   return $self->SUPER::isa(@_)
-    unless ref $self;
+    if !ref $self or $Moo::sification::disabled;
   require Moo::HandleMoose;
   Moo::HandleMoose::inject_real_metaclass_for($self->{name})->isa(@_)
 }
