@@ -1,6 +1,5 @@
-use strict;
-use warnings;
-
+use Moo::_strictures;
+use lib 'lib';
 {
   package BaseClass;
   use Moo;
